package it.arquillian.moduleTest.jpa.controller;

import java.lang.reflect.Field;
import java.util.List;
import org.junit.rules.MethodRule;
import org.junit.runners.model.FrameworkMethod;
import org.junit.runners.model.Statement;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Set Parametric Test ActionJpaController EJB
 * 
 * https://gist.github.com/aslakknutsen/1358803
 * 
 * @author Marco!Cella
 */
public class ActionJpaControllerParameterRule implements MethodRule {
    
    private static String className = ActionJpaControllerParameterRule.class.getName();
    
    final Logger logger = LoggerFactory.getLogger(ActionJpaControllerParameterRule.class);
    
    private final List<String> fieldNames;
    private final List<List<Object>> params;
    
    public ActionJpaControllerParameterRule( List<String> fieldNames, List<List<Object>> params ) {
        if(fieldNames == null) {
            throw new IllegalArgumentException("fieldNames must be specified");
        }
        if(params == null) {
            throw new IllegalArgumentException("params must be specified");
        }
        this.fieldNames = fieldNames;
        this.params = params;
    }
    
    @Override
    public Statement apply(final Statement base, final FrameworkMethod method, final Object target) {
        return new Statement() {
            @Override
            public void evaluate() throws Throwable {
                logger.info("rule - before {0}", target.hashCode());
                if(isInContainer()) {
                    for(List<Object> list: params) {
                        for(int i=0; i<fieldNames.size(); i++) {
                            String fieldName = fieldNames.get(i);
                            Field targetField = target.getClass().getDeclaredField(fieldName);
                            if(!targetField.isAccessible()) {
                                targetField.setAccessible(true);
                            }
                            Object param = list.get(i);
                            targetField.set(target, param);
                        }
                        base.evaluate();
                    }           
                } else {
                    base.evaluate();
                }
                logger.info("rule - after {0}", target.hashCode());
            }
        };
    }

    private boolean isInContainer() {
        Exception e = new Exception();
        e.fillInStackTrace();
        return e.getStackTrace()[e.getStackTrace().length-1].getClassName().equals("java.lang.Thread");
    }
    
}
