package it.arquillian.moduleTest.jpa.controller;

import it.arquillian.moduleTest.jpa.ArchiveFactory;
import it.arquillian.moduleTest.jpa.entity.Action;
import it.arquillian.moduleTest.jpa.util.ManageExceptionStackTrace;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import javax.ejb.EJB;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.persistence.*;
import static org.junit.Assert.*;
import org.junit.Rule;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Test parametric ActionJpaController EJB
 *
 * @author Marco!Cella
 */
@RunWith(Arquillian.class)
public class ActionJpaControllerParametricTest extends ArchiveFactory {
    @EJB
    private ActionJpaController actionJpaController;
    
    private static String className = ActionJpaControllerParametricTest.class.getName();
    
    final Logger logger = LoggerFactory.getLogger(ActionJpaControllerParametricTest.class);
    
    private String name;
    private String description;
    private Class exception; 
    
    @Rule
    public ActionJpaControllerParameterRule rule = new ActionJpaControllerParameterRule ( 
            new ArrayList<String>( Arrays.asList("name", "description", "exception") ), 
            new ArrayList<List<Object>>() {{ 
                add( new ArrayList<Object>( Arrays.asList( null, "action_createActionTest_description", javax.validation.ConstraintViolationException.class ) ) );
                add( new ArrayList<Object>( Arrays.asList( "", "action_createActionTest_description", javax.validation.ConstraintViolationException.class ) ) );
                add( new ArrayList<Object>( Arrays.asList( "action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name", "action_createActionTest_description", javax.validation.ConstraintViolationException.class ) ) );
            }} 
    );
    
    @Test
    @Cleanup(phase = TestExecutionPhase.NONE)
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/NotCreateActionAssert.xml")
    public void shouldNotCreateActionParametric() throws Exception {
        logger.info("\nExecute test: {} [#] shouldNotCreateActionParametric", className);
        // given
        logger.info("\ngivenName={}", name);
        logger.info("\ngivenDescription={}", description);
        String givenName = name;
        String givenDescription = description;
        Action givenAction = new Action();
        Class expectedExceptionClass = exception;
        boolean testStatus = false;
        String actualException = null;
        try {
            // when
            givenAction.setName( givenName );
            givenAction.setDescription( givenDescription );
            actionJpaController.create( givenAction );
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            actualException = (!testStatus) ? ManageExceptionStackTrace.getInnerException(ex) : null;
        }
        assertTrue( "\nExpected: " + expectedExceptionClass + "\nGet: " + actualException, testStatus );
    }
    
}
