package it.arquillian.moduleTest.jpa.controller;

import it.arquillian.moduleTest.jpa.ArchiveFactory;
import it.arquillian.moduleTest.jpa.entity.Action;
import it.arquillian.moduleTest.jpa.exceptions.PreexistingEntityException;
import it.arquillian.moduleTest.jpa.exceptions.RollbackFailureException;
import it.arquillian.moduleTest.jpa.util.ManageExceptionStackTrace;
import javax.ejb.EJB;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.junit.InSequence;
import org.jboss.arquillian.persistence.*;
import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Test ActionJpaController EJB
 *
 * @author Marco!Cella
 */
@RunWith(Arquillian.class)
public class ActionJpaControllerTest extends ArchiveFactory {
    @EJB
    private ActionJpaController actionJpaController;
    
    private static String className = ActionJpaControllerTest.class.getName();
    
    final Logger logger = LoggerFactory.getLogger(ActionJpaControllerTest.class);
    
    @Test
    @Cleanup(phase = TestExecutionPhase.NONE)
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/NotCreateActionAssert.xml")
    public void shouldNotCreateActionOnNullNameField() throws Exception {
        logger.info("\nExecute test: {} [#] shouldNotCreateActionOnNullNameField", className);
        // given
        Action givenAction = new Action();
        Class expectedExceptionClass = javax.validation.ConstraintViolationException.class;
        boolean testStatus = false;
        String actualException = null;
        try {
            // when
            givenAction.setName( null );
            givenAction.setDescription( "action_createActionTest_description" );
            actionJpaController.create( givenAction );
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            actualException = (!testStatus) ? ManageExceptionStackTrace.getInnerException(ex) : null;
        }
        assertTrue( "\nExpected: " + expectedExceptionClass + "\nGet: " + actualException, testStatus );
    }
    
    @Test
    @Cleanup(phase = TestExecutionPhase.NONE)
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/NotCreateActionAssert.xml")
    public void shouldNotCreateActionOnNotReachedMinimumLengthNameField() throws Exception {
        logger.info("\nExecute test: {} [#] shouldNotCreateActionOnNotReachedMinimumLengthNameField", className);
        // given
        Action givenAction = new Action();
        Class expectedExceptionClass = javax.validation.ConstraintViolationException.class;
        boolean testStatus = false;
        String actualException = null;
        try {
            // when
            givenAction.setName( "" );
            givenAction.setDescription( "action_createActionTest_description" );
            actionJpaController.create( givenAction );
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            actualException = (!testStatus) ? ManageExceptionStackTrace.getInnerException(ex) : null;
        }
        assertTrue( "\nExpected: " + expectedExceptionClass + "\nGet: " + actualException, testStatus );
    }
    
    @Test
    @Cleanup(phase = TestExecutionPhase.NONE)
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/NotCreateActionAssert.xml")
    public void shouldNotCreateActionOnExceedingLengthNameField() throws Exception {
        logger.info("\nExecute test: {} [#] shouldNotCreateActionOnExceedingLengthNameField", className);
        // given
        Action givenAction = new Action();
        Class expectedExceptionClass = javax.validation.ConstraintViolationException.class;
        boolean testStatus = false;
        String actualException = null;
        try {
            // when
            givenAction.setName( "action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name_action_createActionTest_name" );
            givenAction.setDescription( "action_createActionTest_description" );
            actionJpaController.create( givenAction );
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            actualException = (!testStatus) ? ManageExceptionStackTrace.getInnerException(ex) : null;
        }
        assertTrue( "\nExpected: " + expectedExceptionClass + "\nGet: " + actualException, testStatus );
    }
    
}
